#ifndef DISTRIBUTOR_H
#define DISTRIBUTOR_H

#include <sys/types.h>
#include <deque>
#include <vector>
#include <boost/thread/thread.hpp>
#include <boost/thread/mutex.hpp>
#include <boost/thread/condition.hpp>

#include "Result.h"
#include "Globals.h"

class Buffer;
class Reader;
class WFCounter;

class Distributor
{
public:
    Distributor( const char * filename, int numReader, int numWorker );
    ~Distributor();

    int process();
    void deliverResult( Result * result );
    void deliverResult( Result * result[Last] );
    void readerFinished( Reader * reader );

    void pushBuffer( Buffer * buffer );
    Buffer * popBuffer();

private:
    bool processNext();

    const char * m_filename;
    boost::thread_group m_threads;
    int m_numReader;
    int m_numWorker;
    int m_finished;
    boost::mutex m_finishedMutex;
    boost::condition m_finishedCondition;

    std::deque<Buffer *> m_buffers;
    boost::mutex m_buffersMutex;
    boost::condition m_bufferAvailable;
    int m_finishedReaders;

    std::vector<Reader *> m_readers;
    std::vector<WFCounter *> m_counters;
    std::deque<Result *> m_results[Last];
    Result * m_done[Last];
    boost::mutex m_resultMutex;
};

#endif /* DISTRIBUTOR_H */
