#include "Reader.h"

#include <sys/stat.h>
#include <fcntl.h>
#include "Distributor.h"

#define NUM_OF_BUFFERS 64

Reader::Reader( Distributor * distributor, const char * filename, off64_t start, off64_t stop )
: m_distributor( distributor )
, m_filename( filename )
, m_start( start )
, m_current( start )
, m_stop( stop )
, m_fd( -1 )
{
    for ( int i = 0; i < NUM_OF_BUFFERS; ++i ) {
        m_buffers.push_back( new Buffer( this ) );
    }
}

void Reader::run()
{
    m_fd = open( m_filename, O_RDONLY|O_LARGEFILE );
    if ( m_fd == -1 ) {
        exit( -1 );
    }

    Buffer * buffer = popBuffer();

    if ( m_start > 0 ) {
        char * lineStart = buffer->data;
        lseek64( m_fd, m_start - 1, SEEK_SET );
        fillBuffer( buffer );
        lineStart = std::find( buffer->data, buffer->data + buffer->size, '\n' );
        ++lineStart;
        memmove( buffer->data, lineStart, buffer->size - ( lineStart - buffer->data ) );
    }
    else {
        fillBuffer( buffer );
    }

    const char * end = "\n";
    const char * endend = end + 1;

    while ( 1 ) {
        if ( m_current >= m_stop ) {
            // find first lineEnd starting at pos stop
            char * lineEnd = std::find( buffer->data + buffer->size - ( m_current - m_stop + 1 ), buffer->data + buffer->size, '\n' );
            // if not find read once more
            if ( lineEnd != buffer->data + buffer->size ) {
                // reduce size
                buffer->size = lineEnd - buffer->data;
                // pushbuffer
                m_distributor->pushBuffer( buffer );
                break;
            }
        }
        char * lineEnd = std::find_end( buffer->data, buffer->data + buffer->size, end, endend );
        // move to a new buffer
        Buffer * newBuffer( popBuffer() );
        newBuffer->size = buffer->data + buffer->size - lineEnd - 1;
        memcpy( newBuffer->data, lineEnd + 1, newBuffer->size );
        // push buffer to distributor
        buffer->size -= newBuffer->size;
        m_distributor->pushBuffer( buffer );
        buffer = newBuffer;
        if ( fillBuffer( buffer ) ) {
            break;
        }
    }
    m_distributor->readerFinished( this );

    close( m_fd );
}

void Reader::pushBuffer( Buffer * buffer )
{
    boost::mutex::scoped_lock lock( m_buffersMutex );
    m_buffers.push_back( buffer );
    m_bufferAvailable.notify_one();
}

Buffer * Reader::popBuffer()
{
    boost::mutex::scoped_lock lock( m_buffersMutex );
    while ( m_buffers.empty() ) {
        m_bufferAvailable.wait( lock );
    }

    Buffer * buffer = m_buffers.front();
    m_buffers.pop_front();
    return buffer;
}

bool Reader::fillBuffer( Buffer * buffer )
{
    ssize_t bytes = read( m_fd, buffer->data + buffer->size, BUFFERSIZE - buffer->size );
    buffer->size = bytes + buffer->size;
    m_current += bytes;
    return bytes == 0;
}
