#ifndef READER_H
#define READER_H

#include <deque>
#include <boost/thread/thread.hpp>
#include <boost/thread/mutex.hpp>
#include <boost/thread/condition.hpp>
#include <sys/types.h>

#define BUFFERSIZE 32768
//#define BUFFERSIZE 16384

class Reader;
class Distributor;

struct Buffer
{
    Buffer( Reader * reader ) : reader( reader ), size( 0 ) {}

    Reader * reader;
    char data[BUFFERSIZE];
    ssize_t size;
};

class Reader
{
public:
    Reader( Distributor * distributor, const char * filename, off64_t start, off64_t stop );
    void run();

    void pushBuffer( Buffer * buffer );

private:
    Buffer * popBuffer();
    bool fillBuffer( Buffer * buffer );

    Distributor * m_distributor;
    const char * m_filename;
    off64_t m_start;
    off64_t m_current;
    off64_t m_stop;

    std::deque<Buffer *> m_buffers;
    boost::mutex m_buffersMutex;
    boost::condition m_bufferAvailable;

    int m_fd;
};

#endif /* READER_H */
