#include "Result.h"

static const struct {
    const char * lable;
    bool shrink;
} lables[] = {
    { "client addresses", false },
    { "404s", false },
    { "referrers", false },
    { "URIs by hit", false },
    { "URIs by bytes", true }
};

static const unsigned int numOfResults = sizeof( lables ) / sizeof( *lables );

Result::Result( Type type )
: type( type )
, count( 1 )
{
}

void Result::report() const
{
    printf( "Top %s:\n", lables[type].lable );

    int count = 0;
    int64_t min = 0;

    std::multimap<int64_t, std::string> result;
    for ( std::map<std::string, int64_t>::const_iterator it( data.begin() ); it != data.end(); ++it ) {
        if ( count == 10 ) {
            if ( min < it->second ) {
                result.erase( result.begin() );
                result.insert( std::pair<int64_t, std::string>( it->second, it->first ) );
                min = result.begin()->first;
            }
        }
        else {
            ++count;
            result.insert( std::pair<int64_t, std::string>( it->second, it->first ) );
            if ( min < it->second ) {
                min = it->second;
            }
        }
    }

    const char * fmt = ( lables[type].shrink ) ? " %9.1fM: %s\n" : " %10.0f: %s\n";

    for ( std::multimap<int64_t, std::string>::reverse_iterator it( result.rbegin() ); it != result.rend(); ++it ) {
        std::string pkey = ( it->second.length() > 60 ) ? it->second.substr( 0, 59 ) + "..." : it->second;
        printf( fmt, lables[type].shrink ? it->first / ( 1024.0 * 1024.0 ) : it->first, pkey.c_str() );
    }
    printf( "\n" );
}

void Result::merge( Result * other )
{
    std::map<std::string, int64_t>::iterator it( data.begin() );
    std::map<std::string, int64_t>::const_iterator otherIt( other->data.begin() );

    while ( otherIt != other->data.end() ) {
        if ( it != data.end() ) {
            if ( it->first > otherIt->first ) {
                data.insert( it, *otherIt );
                ++otherIt;
            }
            else if ( it->first < otherIt->first ) {
                ++it;
            }
            else {
                it->second += otherIt->second;
                ++it;
                ++otherIt;
            }
        }
        else {
            for ( ; otherIt != other->data.end(); ++otherIt ) {
                it = data.insert( it, *otherIt );
            }
            break;
        }
    }
    count += other->count;
    //delete other;
}


