#include "WFCounter.h"
#include <iostream>

#include "Distributor.h"
#include "LineData.h"
#include "Result.h"
#include "Reader.h"

WFCounter::WFCounter( Distributor * distributor )
: m_distributor( distributor )
{
    for ( int i = 0; i < Last; ++i ) {
        m_result[i] = new Result( Type( i ) );
    }
    if ( regcomp( &m_exp, "^/ongoing/When/[0-9]{3}x/[0-9]{4}/[0-9]{2}/[0-9]{2}/[^ .]+$", REG_NOSUB | REG_EXTENDED ) ) {
        std::cerr << "regcomp nok" << std::endl;
        exit( 1 );
    }

    if ( regcomp( &m_expRef, "^(-$|http://www.tbray.org/ongoing/)", REG_NOSUB | REG_EXTENDED  ) ) {
        std::cerr << "regcomp nok" << std::endl;
        exit( 1 );
    }
}

void WFCounter::run()
{
    while ( Buffer * buffer = m_distributor->popBuffer() ) {
        char * lineStart = buffer->data;
        const char * bufferEnd = buffer->data + buffer->size;

        while ( lineStart < bufferEnd ) {
            char * lineEnd = std::find( lineStart, buffer->data + buffer->size, '\n' );
            ssize_t length = lineEnd - lineStart + 1;
            process( lineStart, length );
            lineStart += length;
        }
        buffer->reader->pushBuffer( buffer );
    }

    m_distributor->deliverResult( m_result );
}

void WFCounter::process( char * line, ssize_t length )
{
    LineData ld( line, length );
    if ( ld.m_uri != 0 ) {
        switch ( ld.m_status ) {
            case 200:
                m_result[Bytes]->data[ld.m_uri] += ld.m_bytes;
            case 304:
            {
                int count = 0;
                regmatch_t matchOffset[count];
                if ( regexec( &m_exp, ld.m_uri, count, matchOffset, 0 ) == 0 ) {
                    m_result[Hits]->data[ld.m_uri] += 1;
                    m_result[Clients]->data[ld.m_client] += 1;
                    if ( regexec( &m_expRef, ld.m_ref, count, matchOffset, 0 ) != 0 ) {
                        m_result[Refs]->data[ld.m_ref] += 1;
                    }
                }
            }   break;
            case 404:
                m_result[FourOFours]->data[ld.m_uri] += 1;
                break;
            default:
                break;
        }
    }
}
